package javax.swing;

import javax.accessibility.AccessibleSelection;
import javax.swing.plaf.basic.BasicComboPopup;
import javax.accessibility.AccessibleState;
import java.beans.PropertyChangeEvent;
import javax.accessibility.AccessibleRole;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import javax.swing.event.MenuKeyEvent;
import java.awt.event.MouseEvent;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.Serializable;
import java.io.ObjectOutputStream;
import javax.accessibility.AccessibleContext;
import java.awt.Graphics;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.MenuKeyListener;
import javax.swing.event.PopupMenuListener;
import java.util.Iterator;
import java.util.Vector;
import java.beans.PropertyChangeListener;
import sun.awt.SunToolkit;
import java.awt.GraphicsDevice;
import java.awt.Dimension;
import java.awt.GraphicsConfiguration;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.GraphicsEnvironment;
import java.awt.Point;
import java.awt.event.KeyEvent;
import java.awt.event.FocusEvent;
import javax.swing.plaf.PopupMenuUI;
import java.awt.Insets;
import java.awt.Frame;
import java.awt.Component;
import javax.accessibility.Accessible;

public class JPopupMenu extends JComponent implements Accessible, MenuElement
{
    private static final String uiClassID = "PopupMenuUI";
    private static final Object defaultLWPopupEnabledKey;
    static boolean popupPostionFixDisabled;
    transient Component invoker;
    transient Popup popup;
    transient Frame frame;
    private int desiredLocationX;
    private int desiredLocationY;
    private String label;
    private boolean paintBorder;
    private Insets margin;
    private boolean lightWeightPopup;
    private SingleSelectionModel selectionModel;
    private static final Object classLock;
    private static final boolean TRACE = false;
    private static final boolean VERBOSE = false;
    private static final boolean DEBUG = false;
    
    public static void setDefaultLightWeightPopupEnabled(boolean b) {
        SwingUtilities.appContextPut(JPopupMenu.defaultLWPopupEnabledKey, Boolean.valueOf(b));
    }
    
    public static boolean getDefaultLightWeightPopupEnabled() {
        Boolean b = (Boolean)SwingUtilities.appContextGet(JPopupMenu.defaultLWPopupEnabledKey);
        if (b == null) {
            SwingUtilities.appContextPut(JPopupMenu.defaultLWPopupEnabledKey, Boolean.TRUE);
            return true;
        }
        return (boolean)b;
    }
    
    public JPopupMenu() {
        this(null);
    }
    
    public JPopupMenu(String label) {
        this.label = null;
        this.paintBorder = true;
        this.margin = null;
        this.lightWeightPopup = true;
        this.label = label;
        this.lightWeightPopup = getDefaultLightWeightPopupEnabled();
        this.setSelectionModel(new DefaultSingleSelectionModel());
        this.enableEvents(16L);
        this.setFocusTraversalKeysEnabled(false);
        this.updateUI();
    }
    
    public PopupMenuUI getUI() {
        return (PopupMenuUI)this.ui;
    }
    
    public void setUI(PopupMenuUI ui) {
        super.setUI(ui);
    }
    
    @Override
    public void updateUI() {
        this.setUI((PopupMenuUI)UIManager.getUI(this));
    }
    
    @Override
    public String getUIClassID() {
        return "PopupMenuUI";
    }
    
    @Override
    protected void processFocusEvent(FocusEvent e) {
        super.processFocusEvent(e);
    }
    
    @Override
    protected void processKeyEvent(KeyEvent keyEvent) {
        MenuSelectionManager.defaultManager().processKeyEvent(keyEvent);
        if (keyEvent.isConsumed()) {
            return;
        }
        super.processKeyEvent(keyEvent);
        return;
    }
    
    public SingleSelectionModel getSelectionModel() {
        return this.selectionModel;
    }
    
    public void setSelectionModel(SingleSelectionModel selectionModel) {
        this.selectionModel = selectionModel;
    }
    
    public JMenuItem add(JMenuItem comp) {
        super.add(comp);
        return comp;
    }
    
    public JMenuItem add(String s) {
        return this.add(new JMenuItem(s));
    }
    
    public JMenuItem add(Action action) {
        JMenuItem actionComponent = this.createActionComponent(action);
        actionComponent.setAction(action);
        this.add(actionComponent);
        return actionComponent;
    }
    
    Point adjustPopupLocationToFitScreen(int x, int y) {
        Point point = new Point(x, y);
        if (JPopupMenu.popupPostionFixDisabled || GraphicsEnvironment.isHeadless()) {
            return point;
        }
        GraphicsConfiguration currentGraphicsConfiguration = this.getCurrentGraphicsConfiguration(point);
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        Rectangle bounds;
        if (currentGraphicsConfiguration != null) {
            bounds = currentGraphicsConfiguration.getBounds();
        }
        else {
            bounds = new Rectangle(defaultToolkit.getScreenSize());
        }
        Dimension preferredSize = this.getPreferredSize();
        long n = (long)point.x + (long)preferredSize.width;
        long n2 = (long)point.y + (long)preferredSize.height;
        int width = bounds.width;
        int height = bounds.height;
        if (!canPopupOverlapTaskBar()) {
            Insets screenInsets = defaultToolkit.getScreenInsets(currentGraphicsConfiguration);
            bounds.x += screenInsets.left;
            bounds.y += screenInsets.top;
            width -= screenInsets.left + screenInsets.right;
            height -= screenInsets.top + screenInsets.bottom;
        }
        int n3 = bounds.x + width;
        int n4 = bounds.y + height;
        if (n > (long)n3) {
            point.x = n3 - preferredSize.width;
        }
        if (n2 > (long)n4) {
            point.y = n4 - preferredSize.height;
        }
        if (point.x < bounds.x) {
            point.x = bounds.x;
        }
        if (point.y < bounds.y) {
            point.y = bounds.y;
            return point;
        }
        return point;
    }
    
    private GraphicsConfiguration getCurrentGraphicsConfiguration(Point p) {
        GraphicsConfiguration graphicsConfiguration = null;
        GraphicsDevice[] screenDevices = GraphicsEnvironment.getLocalGraphicsEnvironment().getScreenDevices();
        for (int i = 0; i < screenDevices.length; ++i) {
            if (screenDevices[i].getType() == 0) {
                GraphicsConfiguration defaultConfiguration = screenDevices[i].getDefaultConfiguration();
                if (defaultConfiguration.getBounds().contains(p)) {
                    graphicsConfiguration = defaultConfiguration;
                    break;
                }
            }
        }
        if (graphicsConfiguration == null && this.getInvoker() != null) {
            graphicsConfiguration = this.getInvoker().getGraphicsConfiguration();
            return graphicsConfiguration;
        }
        return graphicsConfiguration;
    }
    
    static boolean canPopupOverlapTaskBar() {
        boolean canPopupOverlapTaskBar = true;
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        if ((defaultToolkit instanceof SunToolkit)) {
            canPopupOverlapTaskBar = ((SunToolkit)defaultToolkit).canPopupOverlapTaskBar();
        }
        return canPopupOverlapTaskBar;
    }
    
    protected JMenuItem createActionComponent(Action action) {
        JMenuItem menuItem = new JMenuItem() {
            @Override
            protected PropertyChangeListener createActionPropertyChangeListener(Action action) {
                PropertyChangeListener propertyChangeListener = JPopupMenu.this.createActionChangeListener(this);
                if (propertyChangeListener == null) {
                    propertyChangeListener = super.createActionPropertyChangeListener(action);
                    return propertyChangeListener;
                }
                return propertyChangeListener;
            }
        };
        menuItem.setHorizontalTextPosition(11);
        menuItem.setVerticalTextPosition(0);
        return menuItem;
    }
    
    protected PropertyChangeListener createActionChangeListener(JMenuItem menuItem) {
        return menuItem.createActionPropertyChangeListener0(menuItem.getAction());
    }
    
    @Override
    public void remove(int index) {
        if (index < 0) {
            throw new IllegalArgumentException("index less than zero.");
        }
        if (index > this.getComponentCount() - 1) {
            throw new IllegalArgumentException("index greater than the number of items.");
        }
        super.remove(index);
        return;
    }
    
    public void setLightWeightPopupEnabled(boolean lightWeightPopup) {
        this.lightWeightPopup = lightWeightPopup;
    }
    
    public boolean isLightWeightPopupEnabled() {
        return this.lightWeightPopup;
    }
    
    public String getLabel() {
        return this.label;
    }
    
    public void setLabel(String s) {
        String label = this.label;
        this.firePropertyChange("label", label, this.label = s);
        if (this.accessibleContext != null) {
            this.accessibleContext.firePropertyChange("AccessibleVisibleData", label, s);
        }
        this.invalidate();
        this.repaint();
    }
    
    public void addSeparator() {
        this.add(new JPopupMenu.Separator());
    }
    
    public void insert(Action action, int n) {
        JMenuItem actionComponent = this.createActionComponent(action);
        actionComponent.setAction(action);
        this.insert(actionComponent, n);
    }
    
    public void insert(Component comp, int i) {
        if (i < 0) {
            throw new IllegalArgumentException("index less than zero.");
        }
        int componentCount = this.getComponentCount();
        Vector<Component> vector = new Vector();
        for (; i < componentCount; ++i) {
            vector.addElement(this.getComponent(i));
            this.remove(i);
        }
        this.add(comp);
        Iterator<Object> iterator = vector.iterator();
        while (iterator.hasNext()) {
            this.add((Component)iterator.next());
        }
        return;
    }
    
    public void addPopupMenuListener(PopupMenuListener l) {
        this.listenerList.add(PopupMenuListener.class, l);
    }
    
    public void removePopupMenuListener(PopupMenuListener l) {
        this.listenerList.remove(PopupMenuListener.class, l);
    }
    
    public PopupMenuListener[] getPopupMenuListeners() {
        return (PopupMenuListener[])this.listenerList.getListeners(PopupMenuListener.class);
    }
    
    public void addMenuKeyListener(MenuKeyListener l) {
        this.listenerList.add(MenuKeyListener.class, l);
    }
    
    public void removeMenuKeyListener(MenuKeyListener l) {
        this.listenerList.remove(MenuKeyListener.class, l);
    }
    
    public MenuKeyListener[] getMenuKeyListeners() {
        return (MenuKeyListener[])this.listenerList.getListeners(MenuKeyListener.class);
    }
    
    protected void firePopupMenuWillBecomeVisible() {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == PopupMenuListener.class) {
                if (o == null) {
                    o = new PopupMenuEvent(this);
                }
                ((PopupMenuListener)listenerList[i + 1]).popupMenuWillBecomeVisible(o);
            }
        }
    }
    
    protected void firePopupMenuWillBecomeInvisible() {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == PopupMenuListener.class) {
                if (o == null) {
                    o = new PopupMenuEvent(this);
                }
                ((PopupMenuListener)listenerList[i + 1]).popupMenuWillBecomeInvisible(o);
            }
        }
    }
    
    protected void firePopupMenuCanceled() {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == PopupMenuListener.class) {
                if (o == null) {
                    o = new PopupMenuEvent(this);
                }
                ((PopupMenuListener)listenerList[i + 1]).popupMenuCanceled(o);
            }
        }
    }
    
    @Override
    boolean alwaysOnTop() {
        return true;
    }
    
    public void pack() {
        if (this.popup != null) {
            Dimension preferredSize = this.getPreferredSize();
            if (preferredSize == null || preferredSize.width != this.getWidth() || preferredSize.height != this.getHeight()) {
                this.showPopup();
            }
            else {
                this.validate();
            }
        }
    }
    
    @Override
    public void setVisible(boolean b) {
        if (b == this.isVisible()) {
            return;
        }
        if (!b) {
            Boolean b2 = (Boolean)this.getClientProperty("JPopupMenu.firePopupMenuCanceled");
            if (b2 != null && b2 == Boolean.TRUE) {
                this.putClientProperty("JPopupMenu.firePopupMenuCanceled", Boolean.FALSE);
                this.firePopupMenuCanceled();
            }
            this.getSelectionModel().clearSelection();
        }
        else if (this.isPopupMenu()) {
            MenuSelectionManager.defaultManager().setSelectedPath(new MenuElement[] { this });
        }
        if (b) {
            this.firePopupMenuWillBecomeVisible();
            this.showPopup();
            this.firePropertyChange("visible", Boolean.FALSE, Boolean.TRUE);
        }
        else if (this.popup != null) {
            this.firePopupMenuWillBecomeInvisible();
            this.popup.hide();
            this.popup = null;
            this.firePropertyChange("visible", Boolean.TRUE, Boolean.FALSE);
            if (this.isPopupMenu()) {
                MenuSelectionManager.defaultManager().clearSelectedPath();
            }
        }
        return;
    }
    
    private void showPopup() {
        Popup popup = this.popup;
        if (popup != null) {
            popup.hide();
        }
        PopupFactory sharedInstance = PopupFactory.getSharedInstance();
        if (this.isLightWeightPopupEnabled()) {
            sharedInstance.setPopupType(0);
        }
        else {
            sharedInstance.setPopupType(2);
        }
        Point adjustPopupLocationToFitScreen = this.adjustPopupLocationToFitScreen(this.desiredLocationX, this.desiredLocationY);
        this.desiredLocationX = adjustPopupLocationToFitScreen.x;
        this.desiredLocationY = adjustPopupLocationToFitScreen.y;
        Popup popup2 = this.getUI().getPopup(this, this.desiredLocationX, this.desiredLocationY);
        sharedInstance.setPopupType(0);
        (this.popup = popup2).show();
    }
    
    @Override
    public boolean isVisible() {
        return this.popup != null;
    }
    
    @Override
    public void setLocation(int desiredLocationX, int desiredLocationY) {
        int desiredLocationX2 = this.desiredLocationX;
        int desiredLocationY2 = this.desiredLocationY;
        this.desiredLocationX = desiredLocationX;
        this.desiredLocationY = desiredLocationY;
        if (this.popup != null && (desiredLocationX != desiredLocationX2 || desiredLocationY != desiredLocationY2)) {
            this.showPopup();
        }
    }
    
    private boolean isPopupMenu() {
        return this.invoker != null && !(this.invoker instanceof JMenu);
    }
    
    public Component getInvoker() {
        return this.invoker;
    }
    
    public void setInvoker(Component invoker) {
        Component invoker2 = this.invoker;
        this.invoker = invoker;
        if (invoker2 != this.invoker && this.ui != null) {
            this.ui.uninstallUI(this);
            this.ui.installUI(this);
        }
        this.invalidate();
    }
    
    public void show(Component invoker, int n, int n2) {
        this.setInvoker(invoker);
        Frame frame = getFrame(invoker);
        if (frame != this.frame && frame != null) {
            this.frame = frame;
            if (this.popup != null) {
                this.setVisible(false);
            }
        }
        if (invoker != null) {
            Point locationOnScreen = invoker.getLocationOnScreen();
            long n3 = (long)locationOnScreen.x + (long)n;
            long n4 = (long)locationOnScreen.y + (long)n2;
            if (n3 > 2147483647L) {
                n3 = 2147483647L;
            }
            if (n3 < -2147483648L) {
                n3 = -2147483648L;
            }
            if (n4 > 2147483647L) {
                n4 = 2147483647L;
            }
            if (n4 < -2147483648L) {
                n4 = -2147483648L;
            }
            this.setLocation((int)n3, (int)n4);
        }
        else {
            this.setLocation(n, n2);
        }
        this.setVisible(true);
    }
    
    JPopupMenu getRootPopupMenu() {
        while (this != null && !this.isPopupMenu() && this.getInvoker() != null && this.getInvoker().getParent() != null && this.getInvoker().getParent() instanceof JPopupMenu) {
            this = (JPopupMenu)this.getInvoker().getParent();
        }
        return this;
    }
    
    @Deprecated
    public Component getComponentAtIndex(int n) {
        return this.getComponent(n);
    }
    
    public int getComponentIndex(Component component) {
        int componentCount = this.getComponentCount();
        Component[] components = this.getComponents();
        int i = 0;
        while (i < componentCount) {
            if (components[i] == component) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    public void setPopupSize(Dimension preferredSize) {
        Dimension preferredSize2 = this.getPreferredSize();
        this.setPreferredSize(preferredSize);
        if (this.popup != null && !preferredSize2.equals(this.getPreferredSize())) {
            this.showPopup();
        }
    }
    
    public void setPopupSize(int width, int height) {
        this.setPopupSize(new Dimension(width, height));
    }
    
    public void setSelected(Component component) {
        this.getSelectionModel().setSelectedIndex(this.getComponentIndex(component));
    }
    
    public boolean isBorderPainted() {
        return this.paintBorder;
    }
    
    public void setBorderPainted(boolean paintBorder) {
        this.paintBorder = paintBorder;
        this.repaint();
    }
    
    @Override
    protected void paintBorder(Graphics graphics) {
        if (this.isBorderPainted()) {
            super.paintBorder(graphics);
        }
    }
    
    public Insets getMargin() {
        if (this.margin == null) {
            return new Insets(0, 0, 0, 0);
        }
        return this.margin;
    }
    
    boolean isSubPopupMenu(JPopupMenu popupMenu) {
        int componentCount = this.getComponentCount();
        for (Component component : this.getComponents()) {
            if ((component instanceof JMenu)) {
                JPopupMenu popupMenu2 = ((JMenu)component).getPopupMenu();
                if (popupMenu2 == popupMenu) {
                    return true;
                }
                if (popupMenu2.isSubPopupMenu(popupMenu)) {
                    return true;
                }
            }
        }
        return false;
    }
    
    private static Frame getFrame(Component parent) {
        for (; !(parent instanceof Frame) && parent != null; parent = parent.getParent()) {}
        return (Frame)parent;
    }
    
    @Override
    protected String paramString() {
        return super.paramString() + ",desiredLocationX=" + this.desiredLocationX + ",desiredLocationY=" + this.desiredLocationY + ",label=" + ((this.label != null) ? this.label : "") + ",lightWeightPopupEnabled=" + (this.isLightWeightPopupEnabled() ? "true" : "false") + ",margin=" + ((this.margin != null) ? this.margin.toString() : "") + ",paintBorder=" + (this.paintBorder ? "true" : "false");
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JPopupMenu.AccessibleJPopupMenu();
        }
        return this.accessibleContext;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        Vector obj = new Vector();
        objectOutputStream.defaultWriteObject();
        if (this.invoker != null && this.invoker instanceof Serializable) {
            obj.addElement("invoker");
            obj.addElement(this.invoker);
        }
        if (this.popup != null && this.popup instanceof Serializable) {
            obj.addElement("popup");
            obj.addElement(this.popup);
        }
        objectOutputStream.writeObject(obj);
        if (this.getUIClassID().equals("PopupMenuUI")) {
            int n = (byte)(JComponent.getWriteObjCounter(this) - 1);
            JComponent.setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        Vector<Object> vector = (Vector)objectInputStream.readObject();
        int n = 0;
        int size = vector.size();
        if (n < size && vector.elementAt(n).equals("invoker")) {
            this.invoker = (Component)vector.elementAt(++n);
            ++n;
        }
        if (n < size && vector.elementAt(n).equals("popup")) {
            this.popup = (Popup)vector.elementAt(++n);
            ++n;
        }
    }
    
    @Override
    public void processMouseEvent(MouseEvent mouseEvent, MenuElement[] array, MenuSelectionManager menuSelectionManager) {
    }
    
    @Override
    public void processKeyEvent(KeyEvent keyEvent, MenuElement[] p3, MenuSelectionManager m) {
        MenuKeyEvent menuKeyEvent = new MenuKeyEvent(keyEvent.getComponent(), keyEvent.getID(), keyEvent.getWhen(), keyEvent.getModifiers(), keyEvent.getKeyCode(), keyEvent.getKeyChar(), p3, m);
        this.processMenuKeyEvent(menuKeyEvent);
        if (menuKeyEvent.isConsumed()) {
            keyEvent.consume();
        }
    }
    
    private void processMenuKeyEvent(MenuKeyEvent menuKeyEvent) {
        switch (menuKeyEvent.getID()) {
            case 401: {
                this.fireMenuKeyPressed(menuKeyEvent);
                break;
            }
            case 402: {
                this.fireMenuKeyReleased(menuKeyEvent);
                break;
            }
            case 400: {
                this.fireMenuKeyTyped(menuKeyEvent);
                break;
            }
        }
    }
    
    private void fireMenuKeyPressed(MenuKeyEvent menuKeyEvent) {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == MenuKeyListener.class) {
                ((MenuKeyListener)listenerList[i + 1]).menuKeyPressed(menuKeyEvent);
            }
        }
    }
    
    private void fireMenuKeyReleased(MenuKeyEvent menuKeyEvent) {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == MenuKeyListener.class) {
                ((MenuKeyListener)listenerList[i + 1]).menuKeyReleased(menuKeyEvent);
            }
        }
    }
    
    private void fireMenuKeyTyped(MenuKeyEvent menuKeyEvent) {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == MenuKeyListener.class) {
                ((MenuKeyListener)listenerList[i + 1]).menuKeyTyped(menuKeyEvent);
            }
        }
    }
    
    @Override
    public void menuSelectionChanged(boolean b) {
        if ((this.invoker instanceof JMenu)) {
            JMenu menu = (JMenu)this.invoker;
            if (b) {
                menu.setPopupMenuVisible(true);
            }
            else {
                menu.setPopupMenuVisible(false);
            }
        }
        if (this.isPopupMenu() && !b) {
            this.setVisible(false);
        }
    }
    
    @Override
    public MenuElement[] getSubElements() {
        Vector<E> vector = new Vector();
        for (int componentCount = this.getComponentCount(), i = 0; i < componentCount; ++i) {
            Component component = this.getComponent(i);
            if ((component instanceof MenuElement)) {
                vector.addElement((E)component);
            }
        }
        MenuElement[] array = new MenuElement[vector.size()];
        for (int j = 0; j < vector.size(); ++j) {
            array[j] = (MenuElement)vector.elementAt(j);
        }
        return array;
    }
    
    @Override
    public Component getComponent() {
        return this;
    }
    
    public boolean isPopupTrigger(MouseEvent e) {
        return this.getUI().isPopupTrigger(e);
    }
    
    static {
        defaultLWPopupEnabledKey = new StringBuffer("JPopupMenu.defaultLWPopupEnabledKey");
        JPopupMenu.popupPostionFixDisabled = false;
        JPopupMenu.popupPostionFixDisabled = ((String)AccessController.doPrivileged(new GetPropertyAction("javax.swing.adjustPopupLocationToFit", ""))).equals("false");
        classLock = new Object();
    }
    
    protected class AccessibleJPopupMenu extends JComponent.AccessibleJComponent implements PropertyChangeListener
    {
        protected AccessibleJPopupMenu() {
            JPopupMenu.this.addPropertyChangeListener(this);
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.POPUP_MENU;
        }
        
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            if (propertyChangeEvent.getPropertyName() == "visible") {
                if (propertyChangeEvent.getOldValue() == Boolean.FALSE && propertyChangeEvent.getNewValue() == Boolean.TRUE) {
                    this.handlePopupIsVisibleEvent(true);
                }
                else if (propertyChangeEvent.getOldValue() == Boolean.TRUE && propertyChangeEvent.getNewValue() == Boolean.FALSE) {
                    this.handlePopupIsVisibleEvent(false);
                }
            }
        }
        
        private void handlePopupIsVisibleEvent(boolean b) {
            if (b) {
                this.firePropertyChange("AccessibleState", null, AccessibleState.VISIBLE);
                this.fireActiveDescendant();
            }
            else {
                this.firePropertyChange("AccessibleState", AccessibleState.VISIBLE, null);
            }
        }
        
        private void fireActiveDescendant() {
            if ((JPopupMenu.this instanceof BasicComboPopup)) {
                JList<Object> list = ((BasicComboPopup)JPopupMenu.this).getList();
                if (list == null) {
                    return;
                }
                AccessibleSelection accessibleSelection = list.getAccessibleContext().getAccessibleSelection();
                if (accessibleSelection == null) {
                    return;
                }
                Accessible accessibleSelection2 = accessibleSelection.getAccessibleSelection(0);
                if (accessibleSelection2 == null) {
                    return;
                }
                AccessibleContext accessibleContext = accessibleSelection2.getAccessibleContext();
                if (accessibleContext != null && JPopupMenu.this.invoker != null) {
                    AccessibleContext accessibleContext2 = JPopupMenu.this.invoker.getAccessibleContext();
                    if (accessibleContext2 != null) {
                        accessibleContext2.firePropertyChange("AccessibleActiveDescendant", null, accessibleContext);
                    }
                }
            }
        }
    }
    
    public static class Separator extends JSeparator
    {
        public Separator() {
            super(0);
        }
        
        @Override
        public String getUIClassID() {
            return "PopupMenuSeparatorUI";
        }
    }
}
